@@ -1,6 +1,6 @@
 ;;; w3m-ems.el --- GNU Emacs stuff for emacs-w3m
 
-;; Copyright (C) 2001-2011 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2013 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Yuuichi Teranishi  <teranisi@gohome.org>,
 ;;          TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -84,7 +84,9 @@
   (autoload 'w3m-delete-buffer "w3m")
   (autoload 'w3m-image-type "w3m")
   (autoload 'w3m-retrieve "w3m")
-  (autoload 'w3m-select-buffer-update "w3m"))
+  (autoload 'w3m-select-buffer-update "w3m")
+  (unless (fboundp 'image-animate)
+    (defalias 'image-animate 'ignore)))
 
 (eval-and-compile
   (unless (fboundp 'frame-current-scroll-bars)
@@ -235,10 +237,35 @@ This function is an interface to `make-coding-system'."
 circumstances."
   (and w3m-display-inline-images (display-images-p)))
 
+;; Animation.
+(defcustom w3m-image-animate-seconds 10
+  "Animate images (if possible) for this many seconds.
+If nil, don't play the animation.  If t, loop forever."
+  :group 'w3m
+  :type '(choice (integer :tag "Animate for (seconds)")
+		 (const :tag "Inhibit animation" nil)
+		 (const :tag "Animate forever" t)))
+
 (eval-and-compile
-  (defalias 'w3m-ems-create-image (if (fboundp 'create-animated-image)
-				      'create-animated-image
-				    'create-image)))
+  (defalias 'w3m-image-multi-frame-p
+    (if (fboundp 'image-multi-frame-p)
+	'image-multi-frame-p
+      'image-animated-p)))
+
+(defun w3m-image-animate (image)
+  "Start animating IMAGE if possible.  Return IMAGE."
+    (when (and (fboundp 'image-animate)
+	       w3m-image-animate-seconds
+	       (w3m-image-multi-frame-p image))
+      (image-animate image nil w3m-image-animate-seconds)
+      ;; Reset an image to the initial one after playing the animation.
+      ;; FIXME: Is there a better way?
+      (when (numberp w3m-image-animate-seconds)
+	(run-with-timer (1+ w3m-image-animate-seconds) nil
+			(lambda (image)
+			  (image-animate image 0 0))
+			image)))
+    image)
 
 (defun w3m-create-image (url &optional no-cache referer size handler)
   "Retrieve data from URL and create an image object.
@@ -266,7 +293,7 @@ and its cdr element is used as height."
 				    ((match-beginning 2) 'jpeg)
 				    (t 'png)))
 			 (w3m-image-type type))))
-	  (setq image (w3m-ems-create-image
+	  (setq image (create-image
 		       (buffer-string) type t
 		       :ascent 'center
 		       :background w3m-image-default-background))
@@ -290,10 +317,8 @@ and its cdr element is used as height."
 				    (plist-get (cdr image) :data)
 				    (car set-size)(cdr set-size)
 				    handler))
-			(if resized (plist-put (cdr image) :data resized))
-			image))
-		  image))
-	    image))))))
+			(if resized (plist-put (cdr image) :data resized)))))))
+	  (w3m-image-animate image))))))
 
 (defun w3m-create-resized-image (url rate &optional referer size handler)
   "Resize an cached image object.
@@ -432,6 +457,12 @@ Buffer string between BEG and END are replaced with IMAGE."
 		 (insert " ")
 		 (setq start (1+ start)
 		       end (1- end)))))
+	;; Empty text won't be buttonized, so we fill it with something.
+	;; "submit" seems to be a proper choice in nine cases out of ten.
+	(when (= start end)
+	  (goto-char start)
+	  (insert "submit")
+	  (setq end (point)))
 	(let ((w (widget-convert-button
 		  'w3m-form-button start end
 		  :w3m-form-action (plist-get properties 'w3m-action))))
@@ -820,7 +851,8 @@ otherwise works in all the emacs-w3m buffers."
 	     (not (eq (symbol-function 'force-window-update) 'ignore)))
 	(lambda (&optional window) "\
 Force redisplay of WINDOW which defaults to the selected window."
-	  (force-window-update (or window (selected-window))))
+	  (force-window-update (or window (selected-window)))
+	  (sit-for 0))
       (lambda (&optional ignore) "\
 Wobble the selected window to force redisplay of the header-line."
 	(save-window-excursion
@@ -844,13 +876,13 @@ Wobble the selected window to force redisplay of the header-line."
 	(setq window (next-window))))
     (unless (eq (window-buffer window) buffer)
       (select-window window)
-      (w3m-switch-to-buffer buffer)
+      (switch-to-buffer buffer)
       (w3m-force-window-update window))))
 
 (defun w3m-tab-click-mouse-function (event buffer)
   (let ((window (posn-window (event-start event))))
     (select-window window)
-    (w3m-switch-to-buffer buffer)
+    (switch-to-buffer buffer)
     (w3m-force-window-update window)))
 
 (defun w3m-tab-double-click-mouse1-function (event buffer)
@@ -977,7 +1009,7 @@ is non-nil means not to respond to too fast operation of mouse wheel."
   (unless n (setq n 1))
   (when (and (/= n 0) (eq major-mode 'w3m-mode))
     (let ((buffers (w3m-list-buffers)))
-      (w3m-switch-to-buffer
+      (switch-to-buffer
        (nth (mod (+ n (w3m-buffer-number (current-buffer)) -1)
 		 (length buffers))
 	    buffers))
@@ -1033,7 +1065,7 @@ is non-nil means not to respond to too fast operation of mouse wheel."
 	   (f2 (lambda (fn) `(lambda (e)
 			       (interactive "e")
 			       (select-window (posn-window (event-start e)))
-			       (w3m-switch-to-buffer ,cur)
+			       (switch-to-buffer ,cur)
 			       (setq this-command ',fn)
 			       (,fn 1 e))))
 	   (drag-action (funcall f1 'w3m-tab-drag-mouse-function))
